package protocol

import (
    "github.com/iotaledger/goshimmer/packages/identity"
    "github.com/iotaledger/goshimmer/plugins/autopeering/salt"
)

const (
    // PEERING REQUEST PACKET STRUCTURE ////////////////////////////////////////////////////////////////////////////////

    NEIGHBOR_COUNT = 8

    PACKET_HEADER_SIZE = 1
    ISSUER_SIZE        = PEER_MARSHALLED_TOTAL_SIZE
    SALT_SIZE          = salt.SALT_MARSHALLED_SIZE
    SIGNATURE_SIZE     = 65

    PACKET_HEADER_START = 0
    ISSUER_START        = PACKET_HEADER_END
    SALT_START          = ISSUER_END
    SIGNATURE_START     = SALT_END

    PACKET_HEADER_END = PACKET_HEADER_START + PACKET_HEADER_SIZE
    ISSUER_END        = ISSUER_START + ISSUER_SIZE
    SALT_END          = SALT_START + SALT_SIZE
    SIGNATURE_END     = SIGNATURE_START + SIGNATURE_SIZE

    PEERING_REQUEST_MARSHALLED_TOTAL_SIZE = SIGNATURE_END

    PACKET_HEADER = 0xBE

    PEERING_RESPONSE_REJECT = ResponseType(0)
    PEERING_RESPONSE_ACCEPT = ResponseType(1)

    PEERING_RESPONSE_MARSHALLED_PEERS_AMOUNT = NEIGHBOR_COUNT + NEIGHBOR_COUNT*NEIGHBOR_COUNT

    PEERING_RESPONSE_MARSHALLED_TYPE_SIZE      = 1
    PEERING_RESPONSE_MARSHALLED_ISSUER_SIZE    = PEER_MARSHALLED_TOTAL_SIZE
    PEERING_RESPONSE_MARSHALLED_PEER_FLAG_SIZE = 1
    PEERING_RESPONSE_MARSHALLED_PEER_SIZE      = PEERING_RESPONSE_MARSHALLED_PEER_FLAG_SIZE + PEER_MARSHALLED_TOTAL_SIZE
    PEERING_RESPONSE_MARSHALLED_PEERS_SIZE     = PEERING_RESPONSE_MARSHALLED_PEERS_AMOUNT * PEERING_RESPONSE_MARSHALLED_PEER_SIZE
    PEERING_RESPONSE_MARSHALLED_SIGNATURE_SIZE = 65
    PEERING_RESPONSE_MARSHALLED_TOTAL_SIZE     = PEERING_RESPONSE_MARSHALLED_SIGNATURE_END

    PEERING_RESPONSE_MARSHALLED_TYPE_START      = 0
    PEERING_RESPONSE_MARSHALLED_ISSUER_START    = PEERING_RESPONSE_MARSHALLED_TYPE_END
    PEERING_RESPONSE_MARSHALLED_PEERS_START     = PEERING_RESPONSE_MARSHALLED_ISSUER_END
    PEERING_RESPONSE_MARSHALLED_SIGNATURE_START = PEERING_RESPONSE_MARSHALLED_PEERS_END

    PEERING_RESPONSE_MARSHALLED_TYPE_END      = PEERING_RESPONSE_MARSHALLED_TYPE_START + PEERING_RESPONSE_MARSHALLED_TYPE_SIZE
    PEERING_RESPONSE_MARSHALLED_PEERS_END     = PEERING_RESPONSE_MARSHALLED_PEERS_START + PEERING_RESPONSE_MARSHALLED_PEERS_SIZE
    PEERING_RESPONSE_MARSHALLED_ISSUER_END    = PEERING_RESPONSE_MARSHALLED_ISSUER_START + PEERING_RESPONSE_MARSHALLED_ISSUER_SIZE
    PEERING_RESPONSE_MARSHALLED_SIGNATURE_END = PEERING_RESPONSE_MARSHALLED_SIGNATURE_START + PEERING_RESPONSE_MARSHALLED_SIGNATURE_SIZE

    PEER_MARSHALLED_PUBLIC_KEY_SIZE            = identity.PUBLIC_KEY_BYTE_LENGTH
    PEER_MARSHALLED_ADDRESS_TYPE_SIZE          = 1 // ipv4/ipv6
    PEER_MARSHALLED_ADDRESS_SIZE               = 16
    PEER_MARSHALLED_PEERING_PROTOCOL_TYPE_SIZE = 1 // tcp udp
    PEER_MARSHALLED_PEERING_PORT_SIZE          = 2
    PEER_MARSHALLED_GOSSIP_PROTOCOL_TYPE_SIZE  = 1
    PEER_MARSHALLED_GOSSIP_PORT_SIZE           = 2 // tcp udp
    PEER_MARSHALLED_TOTAL_SIZE                 = PEER_MARSHALLED_GOSSIP_PORT_END

    PEER_MARSHALLED_PUBLIC_KEY_START            = 0
    PEER_MARSHALLED_ADDRESS_TYPE_START          = PEER_MARSHALLED_PUBLIC_KEY_END
    PEER_MARSHALLED_ADDRESS_START               = PEER_MARSHALLED_ADDRESS_TYPE_END
    PEER_MARSHALLED_PEERING_PROTOCOL_TYPE_START = PEER_MARSHALLED_ADDRESS_END
    PEER_MARSHALLED_PEERING_PORT_START          = PEER_MARSHALLED_PEERING_PROTOCOL_TYPE_END
    PEER_MARSHALLED_GOSSIP_PROTOCOL_TYPE_START  = PEER_MARSHALLED_PEERING_PORT_END
    PEER_MARSHALLED_GOSSIP_PORT_START           = PEER_MARSHALLED_GOSSIP_PROTOCOL_TYPE_END

    PEER_MARSHALLED_PUBLIC_KEY_END            = PEER_MARSHALLED_PUBLIC_KEY_START + PEER_MARSHALLED_PUBLIC_KEY_SIZE
    PEER_MARSHALLED_ADDRESS_TYPE_END          = PEER_MARSHALLED_ADDRESS_TYPE_START + PEER_MARSHALLED_ADDRESS_TYPE_SIZE
    PEER_MARSHALLED_ADDRESS_END               = PEER_MARSHALLED_ADDRESS_START + PEER_MARSHALLED_ADDRESS_SIZE
    PEER_MARSHALLED_PEERING_PROTOCOL_TYPE_END = PEER_MARSHALLED_PEERING_PROTOCOL_TYPE_START + PEER_MARSHALLED_PEERING_PROTOCOL_TYPE_SIZE
    PEER_MARSHALLED_PEERING_PORT_END          = PEER_MARSHALLED_PEERING_PORT_START + PEER_MARSHALLED_PEERING_PORT_SIZE
    PEER_MARSHALLED_GOSSIP_PROTOCOL_TYPE_END  = PEER_MARSHALLED_GOSSIP_PROTOCOL_TYPE_START + PEER_MARSHALLED_GOSSIP_PROTOCOL_TYPE_SIZE
    PEER_MARSHALLED_GOSSIP_PORT_END           = PEER_MARSHALLED_GOSSIP_PORT_START + PEER_MARSHALLED_GOSSIP_PORT_SIZE

    PEER_MARSHALLED_ADDRESS_TYPE_IPV4 = AddressType(0)
    PEER_MARSHALLED_ADDRESS_TYPE_IPV6 = AddressType(1)

    TCP_PROTOCOL = ProtocolType(0)
    UDP_PROTOCOL = ProtocolType(1)
)
